package com.bpcl.lpg.core.payment.billdesk.impl;

import org.slf4j.LoggerFactory;
import java.security.GeneralSecurityException;
import com.bpcl.lpg.core.model.EasysecsModel;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.Key;
import javax.crypto.spec.SecretKeySpec;
import javax.crypto.Mac;
import java.util.Objects;
import com.bpcl.lpg.core.credentials.LPGCredentialsService;
import org.slf4j.Logger;
import com.bpcl.lpg.core.payment.billdesk.LPGChecksumService;

public class LPGChecksumServiceImpl implements LPGChecksumService
{
    private static final Logger LOG;
    private static final String HMAC_SHA256 = "HmacSHA256";
    private LPGCredentialsService lpgCredentialsService;
    
    public String getChecksumValue(final String stringToEncrypt) {
        final EasysecsModel secretsByKey = this.lpgCredentialsService.getSecretsByKey("billdeskChecksumKey");
        Objects.requireNonNull(secretsByKey.getSecret(), "Checksum secret key shouldn't be null");
        try {
            final Mac hmac = Mac.getInstance("HmacSHA256");
            final SecretKeySpec secretKey = new SecretKeySpec(secretsByKey.getSecret().getBytes(), "HmacSHA256");
            hmac.init(secretKey);
            final byte[] raw = hmac.doFinal(stringToEncrypt.getBytes());
            return this.convertChatToHex(raw);
        }
        catch (final NoSuchAlgorithmException | InvalidKeyException exception) {
            LPGChecksumServiceImpl.LOG.error("Exception while preparing checksum", exception);
            return "";
        }
    }
    
    private String convertChatToHex(final byte[] raw) {
        final StringBuilder createChecksum = new StringBuilder();
        for (final byte b : raw) {
            createChecksum.append(char2hex(b));
        }
        return createChecksum.toString();
    }
    
    public static String char2hex(final byte x) {
        final char[] arr = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
        final char[] c = { arr[(x & 0xF0) >> 4], arr[x & 0xF] };
        return new String(c);
    }
    
    public void setLpgCredentialsService(final LPGCredentialsService lpgCredentialsService) {
        this.lpgCredentialsService = lpgCredentialsService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(LPGChecksumServiceImpl.class);
    }
}
